@@ -3,13 +3,19 @@ from abc import ABC, abstractmethod
 from typing import Union, List, Tuple, Callable
 
 import torch.distributed as torch_distrib
-from torch.utils.data import DataLoader
+from torch.utils.data import DataLoader, RandomSampler
 from torch.utils.data.distributed import DistributedSampler
 
 from pytorch_lightning.core import LightningModule
 from pytorch_lightning.utilities import rank_zero_warn
 from pytorch_lightning.utilities.exceptions import MisconfigurationException
 
+try:
+    from torch.utils.data import IterableDataset
+    ITERABLE_DATASET_EXISTS = True
+except ImportError:
+    ITERABLE_DATASET_EXISTS = False
+
 try:
     from apex import amp
 except ImportError:
@@ -26,6 +32,13 @@ except ImportError:
 else:
     XLA_AVAILABLE = True
 
+try:
+    import horovod.torch as hvd
+except ImportError:
+    HOROVOD_AVAILABLE = False
+else:
+    HOROVOD_AVAILABLE = True
+
 
 def _has_len(dataloader: DataLoader) -> bool:
     """ Checks if a given Dataloader has __len__ method implemented i.e. if
@@ -47,6 +60,7 @@ class TrainerDataLoadingMixin(ABC):
     proc_rank: int
     use_ddp: bool
     use_ddp2: bool
+    use_horovod: bool
     shown_warnings: ...
     val_check_interval: float
     use_tpu: bool
@@ -61,6 +75,7 @@ class TrainerDataLoadingMixin(ABC):
     train_percent_check: float
     val_percent_check: float
     test_percent_check: float
+    replace_sampler_ddp: bool
 
     @abstractmethod
     def is_overriden(self, *args):
@@ -86,12 +101,17 @@ class TrainerDataLoadingMixin(ABC):
     def auto_add_sampler(self, dataloader: DataLoader, train: bool) -> DataLoader:
 
         # don't do anything if it's not a dataloader
-        if not isinstance(dataloader, DataLoader):
+        # don't manipulate iterable datasets
+        is_dataloader = isinstance(dataloader, DataLoader)
+
+        is_iterable_ds = False
+        if ITERABLE_DATASET_EXISTS and hasattr(dataloader, 'dataset'):
+            is_iterable_ds = isinstance(dataloader.dataset, IterableDataset)
+
+        if not is_dataloader or is_iterable_ds:
             return dataloader
-
-        need_dist_sampler = self.use_ddp or self.use_ddp2 or self.use_tpu
-
-        if need_dist_sampler:
+        need_dist_sampler = (self.use_ddp or self.use_ddp2 or self.use_horovod or self.use_tpu)
+        if self.replace_sampler_ddp and need_dist_sampler:
 
             skip_keys = ['sampler', 'batch_sampler', 'dataset_kind']
 
@@ -103,10 +123,23 @@ class TrainerDataLoadingMixin(ABC):
                 sampler = DistributedSampler(
                     dataloader.dataset,
                     num_replicas=xm.xrt_world_size(),
-                    rank=xm.get_ordinal()
+                    rank=xm.get_ordinal(),
                 )
+            elif self.use_horovod:
+                sampler = DistributedSampler(dataloader.dataset,
+                                             num_replicas=hvd.size(),
+                                             rank=hvd.rank())
             else:
-                sampler = DistributedSampler(dataloader.dataset)
+                world_size = {
+                    'ddp': self.num_nodes * self.num_processes,
+                    'ddp2': self.num_nodes,
+                    'ddp_cpu': self.num_processes * self.num_nodes
+                }
+                sampler = DistributedSampler(
+                    dataloader.dataset,
+                    num_replicas=world_size.get(self.distributed_backend, 0),
+                    rank=self.proc_rank,
+                )
 
             dl_args['sampler'] = sampler
             dataloader = type(dataloader)(**dl_args)
@@ -163,8 +196,7 @@ class TrainerDataLoadingMixin(ABC):
                 self.val_check_batch = int(self.num_training_batches * self.val_check_interval)
                 self.val_check_batch = max(1, self.val_check_batch)
 
-    def _reset_eval_dataloader(self, model: LightningModule,
-                               mode: str) -> Tuple[int, List[DataLoader]]:
+    def _reset_eval_dataloader(self, model: LightningModule, mode: str) -> Tuple[int, List[DataLoader]]:
         """Generic method to reset a dataloader for evaluation.
 
         Args:
@@ -179,6 +211,13 @@ class TrainerDataLoadingMixin(ABC):
         if not isinstance(dataloaders, list):
             dataloaders = [dataloaders]
 
+        # shuffling in val and test set is bad practice
+        for loader in dataloaders:
+            if mode in ('val', 'test') and hasattr(loader, 'sampler') and isinstance(loader.sampler, RandomSampler):
+                raise MisconfigurationException(
+                    f'Your {mode}_dataloader has shuffle=True, it is best practice to turn'
+                    ' this off for validation and test dataloaders.')
+
         # add samplers
         dataloaders = [self.auto_add_sampler(dl, train=False) for dl in dataloaders if dl]
 
@@ -247,6 +286,10 @@ class TrainerDataLoadingMixin(ABC):
             # all processes wait until data download has happened
             torch_xla.core.xla_model.rendezvous('pl.TrainerDataLoadingMixin.get_dataloaders')
 
+        elif self.use_horovod:
+            # all processes wait until data download has happened
+            hvd.join()
+
         return dataloader
 
     def determine_data_use_amount(self, train_percent_check: float, val_percent_check: float,
